/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.model.Credentials;
import it.inaf.oats.ia2.tap.utils.tapschema.*;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.attributes.IAjaxCallListener;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.cycle.RequestCycle;

/**
 * SchemaPage class extends the default BasePage class. SchemaPage class has an
 * AjaxTabbedPanel to show the SchemaTabs dinamically, and has the
 * AjaxSubmitLink to submit all the tapschema structure to the tapschema
 * database.
 *
 * @author pietro
 */
public class SchemaPage extends BasePage {

    Form schemaForm;

    final List<ITab> schemataTabs;
    final BootstrapTabPanel schemataTabbedPanel;

    AjaxSubmitLink updateSubmit;
    AjaxSubmitLink backSubmit;
    AjaxSubmitLink exit;

    public TapSchema tapschema;

    /**
     * SchemaPage constructor
     *
     * @param target
     * @param form
     * @param credential
     * @param tapschemaName
     * @param tapschema tapschema structure
     */
    public SchemaPage(
            AjaxRequestTarget target,
            Form<?> form,
            final Credentials credential,
            final String tapschemaName,
            final TapSchema tapschema) {
        super("TapSchema Manager - " + tapschemaName);
        this.tapschema = tapschema;

        //schema form
        schemaForm = new Form("schemaForm");
        add(schemaForm);

        //update AjaxSubmitLink
        updateSubmit = new AjaxSubmitLink(
                "updateSubmit",
                schemaForm) {
                    @Override
                    protected void onError(AjaxRequestTarget target, Form<?> form) {

                    }

                    /**
                     * onSubmit method of updateSubmit AjaxSubmitLink updates
                     * the tapschema database.
                     *
                     * @param target
                     * @param form
                     */
                    @Override
                    protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                        try {
                            UpdateTapSchema updateTapSchema = new UpdateTapSchema();
                            boolean alert = updateTapSchema.controlChecked(target, tapschema);

                            if (!alert) {
                                /*
                                 updateTapSchema.createTapSchemaDatabase(
                                 hostname,
                                 port,
                                 username,
                                 password,
                                 tapschemaName);
                                 */
                                // delete tables in tapschema database
                                updateTapSchema.resetTapSchemaDatabase(
                                        credential,
                                        tapschemaName);

                        // inserts the metadata from the variables of
                                //tapschema structure to the tapschema database
                                updateTapSchema.updateTapSchemaDatabase(
                                        credential,
                                        tapschemaName,
                                        tapschema);
                                addError(tapschemaName + " database updated");
                                //target.appendJavaScript("alert('" + tapschemaName + " database updated')");
                            }

                        } catch (Exception exception) {
                            addError("Exception occurred in updating " + tapschemaName + " database: "
                                    + exception.getMessage());
                            exception.printStackTrace();
                        }
                    }
                };
        updateSubmit.add(new AjaxEventBehavior("onmousedown") {
            @Override
            protected void onEvent(AjaxRequestTarget target) {
                addError("\nUpdating " + tapschemaName + " database...");
            }
        });
        schemaForm.add(updateSubmit);

        //back AjaxSubmitLink
        backSubmit = new AjaxSubmitLink(
                "backSubmit",
                schemaForm) {
                    @Override
                    protected void onError(AjaxRequestTarget target, Form<?> form) {

                    }

                    /**
                     * onSubmit method of backSubmit AjaxSubmitLink sends back
                     * to SelectSchemataPage.
                     *
                     * @param target
                     * @param form
                     */
                    @Override
                    protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                        try {
//                    SelectSchemataPage selectSchemataPage = new SelectSchemataPage(
//                            target,
//                            form,
//                            credential);
//                    setResponsePage(selectSchemataPage);
                        } catch (Exception exception) {
                            addError(exception.getMessage());
                            exception.printStackTrace();
                        }
                    }
                };
        schemaForm.add(backSubmit);

        //close session
        exit = new AjaxSubmitLink(
                "exit",
                schemaForm) {
                    @Override
                    protected void onError(AjaxRequestTarget target, Form<?> form) {
                        addError("Error in closing the session");
                    }

                    /**
                     * onSubmit method of exit AjaxSubmitLink exits from the web
                     * application
                     *
                     * @param target
                     * @param form
                     */
                    @Override
                    protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                        try {
                            setResponsePage(CredentialPage.class);
                            //System.exit(0);
                            this.getSession().invalidate();
                            addError("Session closed");
                        } catch (Exception exception) {
                            addError("Exception occurred in closing session: " + exception.getMessage());
                            exception.printStackTrace();
                        }
                    }
                };
        schemaForm.add(exit);

        //schemata AjaxTabbedPanel
        schemataTabs = new ArrayList<ITab>();
        schemataTabbedPanel = new BootstrapTabPanel("schemataTabbedPanel", schemataTabs);
        schemataTabbedPanel.setOutputMarkupPlaceholderTag(true);
        schemaForm.add(schemataTabbedPanel);
    }

    /**
     * populateSchemaTabs method creates and populates SchemaTabs dinamically.
     *
     * @param target
     * @param form
     * @param credential
     * @param tapschemaName
     * @param tapschema tapschema structure
     */
    public void populateSchemaTabs(
            final AjaxRequestTarget target,
            final Form<?> form,
            final Credentials credential,
            final String tapschemaName,
            final TapSchema tapschema) {
        try {
            //create schemataTabs dynamically
            schemataTabs.removeAll(schemataTabs);
            for (Schema schema : tapschema.schemasMap.values()) {
                String panelId = "panel";

                SchemaTab schemaTab = new SchemaTab(
                        //target,
                        //form,
                        panelId,
                        //credential,
                        //tapschemaName,
                        //tapschema,
                        schema);

                schemataTabs.add(schemaTab);
            }
            target.add(schemataTabbedPanel);
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * populateTableTabs method creates and populates TableTabs dinamically.
     *
     * @param target
     * @param form
     * @param credential
     * @param tapschemaName
     * @param schema current schema structure
     * @param schemaPanel
     */
    public void populateTableTabs(
            AjaxRequestTarget target,
            Form<?> form,
            final Credentials credential,
            final String tapschemaName,
            final Schema schema,
            SchemaPanelOld schemaPanel) {
        try {
            //populates tablesTabs
            List<ITab> tablesTabs = schemaPanel.tablesTabs;
            tablesTabs.removeAll(tablesTabs);
            for (Table table : schema.tablesMap.values()) {
                final String table_name = table.table_name;

                //populates rowsList
                List<Row> rowsList = new ArrayList<Row>();
                rowsList.removeAll(rowsList);
                for (Column column : table.getColumns()) {
                    String column_name = column.getColumnName();

                    String primary_key = "";
                    //if (column.indexed == 1)
                    if (column.getPrincipal() == 1) {
                        primary_key = "pk";
                    }

                    String foreign_key = "";
                    for (Key key : schema.keysMap.values()) {
                        if (key.from_table.equals(table_name)) {
                            for (KeyColumn key_column : schema.key_columnsMap.values()) {
                                if (key_column.key_id.equals(key.key_id)) {
                                    if (key_column.from_column.equals(column_name)) {
                                        foreign_key = "fk";
                                    }
                                }
                            }
                        }
                    }

                    rowsList.add(new Row(
                            column.checked,
                            primary_key,
                            foreign_key,
                            table_name,
                            column_name,
                            column.getUtype(),
                            column.getUcd(),
                            column.getUnit(),
                            column.getDescription(),
                            column.getDatatype(),
                            column.getSize(),
                            column.getPrincipal(),
                            column.getIndexed(),
                            column.getStd()));
                }

                //creates tablesTabs dynamically
                TableTab tableTab = new TableTab(
                        rowsList,
                        credential,
                        tapschemaName,
                        schema,
                        table);

                tablesTabs.add(tableTab);
            }//end for
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

}
