package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.model.ColumnField;
import it.inaf.oats.ia2.tap.utils.model.NullableView;
import it.inaf.oats.ia2.tap.utils.tapschema.Column;
import it.inaf.oats.ia2.tap.utils.tapschema.Schema;
import it.inaf.oats.ia2.tap.utils.tapschema.Table;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.logging.Logger;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.Model;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public final class SchemaPanel extends Panel {

    static final Logger log = Logger.getLogger(SelectSchemataPage.class.getName());

    class ColumnSelector extends AjaxFallbackLink {

        private final ListItem<String> item;

        public ColumnSelector(String id, ListItem<String> item) {
            super(id);
            this.item = item;
            add(new Label("columnName", item.getModel()));
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            SchemaPanel.this.selectColumn(item.getIndex());
            target.add(SchemaPanel.this);
        }
    }

    private final Schema schema;

    Table selectedTable = null;
    final ArrayList<String> tablesNames;
    final Model<String> selectedTableModel;
    final DropDownChoice tableDropDownChoice;

    int selectedColumnIndex = 0;
    final ArrayList<String> columnsNames;
    Column selectedColumn = null;

    final NullableView<Column> selectedColumnView;

    public SchemaPanel(String id, Schema schema) {
        super(id);
        this.schema = schema;
        setOutputMarkupPlaceholderTag(true);

        TextField<String> schemaUtypeTextField
                = new TextField<String>("schemaUtype", new Model<String>() {

                    @Override
                    public String getObject() {
                        return SchemaPanel.this.schema.getUtype();
                    }

                    @Override
                    public void setObject(String object) {
                        SchemaPanel.this.schema.setUtype(object);
                    }
                });
        add(schemaUtypeTextField);

        TextField<String> schemaDescriptionTextField
                = new TextField<String>("schemaDescription", new Model<String>() {

                    @Override
                    public String getObject() {
                        return SchemaPanel.this.schema.getDescription();
                    }

                    @Override
                    public void setObject(String object) {
                        SchemaPanel.this.schema.setDescription(object);
                    }
                });
        add(schemaDescriptionTextField);

        tablesNames = new ArrayList<String>();
        for (String tableName : schema.tablesMap.keySet()) {
            tablesNames.add(tableName);
        }

        selectedTableModel = new Model<String>() {

            private String selectedTableName = null;

            @Override
            public String getObject() {
                return selectedTableName;
            }

            @Override
            public void setObject(String object) {
                selectedTableName = object;
                if (selectedTableName != null) {
                    selectedTable = SchemaPanel.this.schema.getTable(selectedTableName);
                    columnsNames.clear();
                    for (String columnName : selectedTable.getColumnsMap().keySet()) {
                        columnsNames.add(columnName);
                    }

                    selectColumn(0);
                }
            }
        };

        tableDropDownChoice = new DropDownChoice<String>(
                "tableDropDownChoice", selectedTableModel, tablesNames);
        tableDropDownChoice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(SchemaPanel.this);
            }
        });
        add(tableDropDownChoice);

        WebMarkupContainer tableInfo = new WebMarkupContainer("tableInfo");
        tableInfo.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(selectedTableModel.getObject() != null);
            }
        });
        add(tableInfo);

        tableInfo.add(new TextField<String>("tableUtype", new Model<String>() {

            @Override
            public String getObject() {
                if (selectedTable != null) {
                    return selectedTable.getUtype();
                }
                return null;
            }

            @Override
            public void setObject(String object) {
                if (selectedTable != null) {
                    selectedTable.setUtype(object);
                }
            }
        }));
        tableInfo.add(new TextField<String>("tableDescription", new Model<String>() {

            @Override
            public String getObject() {
                if (selectedTable != null) {
                    return selectedTable.getDescription();
                }
                return null;
            }

            @Override
            public void setObject(String object) {
                if (selectedTable != null) {
                    selectedTable.setDescription(object);
                }
            }
        }));

        WebMarkupContainer columnsContainer = new WebMarkupContainer("columnsContainer");
        columnsContainer.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(selectedColumn != null);
            }
        });
        add(columnsContainer);

        columnsNames = new ArrayList<String>();
        columnsContainer.add(new ListView<String>("columnsList", columnsNames) {
            @Override
            protected void populateItem(ListItem<String> item) {
                item.add(new ColumnSelector("columnLink", item));

                if (selectedColumnIndex == item.getIndex()) {
                    item.add(new AttributeModifier("class", "active"));
                }
            }
        });

        selectedColumnView = new NullableView<Column>("selectedColumn") {

            @Override
            protected void populateItem(ListItem<Column> li) {
                li.add(new ColumnPropertyTextField<String>("utype", ColumnField.UTYPE));
                li.add(new ColumnPropertyTextField<String>("ucd", ColumnField.UCD));
                li.add(new ColumnPropertyTextField<String>("unit", ColumnField.UNIT));
                li.add(new ColumnPropertyTextField<String>("description", ColumnField.DESCRIPTION));
                li.add(new ColumnPropertyTextField<String>("datatype", ColumnField.DATATYPE));
            }
        };

        columnsContainer.add(selectedColumnView);
    }

    private void selectColumn(int index) {
        if (columnsNames.size() > 0) {
            selectedColumnIndex = index;
            selectedColumn = selectedTable.getColumn(columnsNames.get(index));
            selectedColumnView.setContent(selectedColumn);
        } else {
            selectedColumnView.clear();
            selectedColumn = null;
        }
    }

    class ColumnPropertyTextField<T extends Serializable> extends TextField<T> {

        protected ColumnPropertyTextField(final String id, final ColumnField key) {
            super(id, selectedColumn.getPropertyModel(key));

            add(new AttributeAppender("class", new Model<String>() {
                @Override
                public String getObject() {
                    return selectedColumn.isChanged(key) ? "changed" : "";
                }
            }, " "));

            // TextField class is updated using JavaScript to avoid it will be 
            // rendered again (and lose caret position inside textfield)
            add(new OnChangeAjaxBehavior() {
                @Override
                protected void onUpdate(final AjaxRequestTarget target) {
                    target.appendJavaScript("setChanged('" + id + "'," + selectedColumn.isChanged(key) + ");");
                }
            });
        }
    }
}
