package it.inaf.oats.ia2.tap.utils.tapschema;

import it.inaf.oats.ia2.tap.utils.model.ColumnField;
import it.inaf.oats.ia2.tap.utils.model.EditableValue;
import java.io.Serializable;
import java.util.HashMap;
import org.apache.wicket.model.Model;

/**
 *
 * @author pietro
 */
public class Column extends Model {

    class ColumnPropertyModel<T extends Serializable> extends Model<T> {

        final ColumnField key;
        
        protected ColumnPropertyModel(ColumnField key) {
            this.key = key;
        }

        @Override
        public T getObject() {
            EditableValue<T> property = properties.get(key);
            if (property != null) {
                return property.getValue();
            }
            return null;
        }

        @Override
        public void setObject(T value) {
            EditableValue<T> property = properties.get(key);
            if (property != null) {
                property.setValue(value);
            }
        }
    }

    private final transient HashMap<ColumnField, EditableValue> properties;

    public Column() {
        properties = new HashMap<ColumnField, EditableValue>();
    }

    private void setValue(ColumnField key, Serializable value) {
        if (properties.containsKey(key)) {
            properties.get(key).setValue(value);
        } else {
            properties.put(key, new EditableValue(value));
        }
    }

    private Object getValue(ColumnField key) {
        return properties.get(key).getValue();
    }

    public boolean checked;

    public String getTableName() {
        return (String) getValue(ColumnField.TABLE_NAME);
    }

    public void setTableName(String tableName) {
        setValue(ColumnField.TABLE_NAME, tableName);
    }

    public String getColumnName() {
        return (String) getValue(ColumnField.COLUMN_NAME);
    }

    public void setColumnName(String columnName) {
        setValue(ColumnField.COLUMN_NAME, columnName);
    }

    public String getUtype() {
        return (String) getValue(ColumnField.UTYPE);
    }

    public void setUtype(String utype) {
        setValue(ColumnField.UTYPE, utype);
    }

    public String getUcd() {
        return (String) getValue(ColumnField.UCD);
    }

    public void setUcd(String ucd) {
        setValue(ColumnField.UCD, ucd);
    }

    public String getUnit() {
        return (String) getValue(ColumnField.UNIT);
    }

    public void setUnit(String unit) {
        setValue(ColumnField.UNIT, unit);
    }

    public String getDescription() {
        return (String) getValue(ColumnField.DESCRIPTION);
    }

    public void setDescription(String description) {
        setValue(ColumnField.DESCRIPTION, description);
    }

    public String getDatatype() {
        return (String) getValue(ColumnField.DATATYPE);
    }

    public void setDatatype(String datatype) {
        setValue(ColumnField.DATATYPE, datatype);
    }

    public int getSize() {
        return (Integer) getValue(ColumnField.SIZE);
    }

    public void setSize(int size) {
        setValue(ColumnField.SIZE, size);
    }

    public int getPrincipal() {
        return (Integer) getValue(ColumnField.PRINCIPAL);
    }

    public void setPrincipal(int principal) {
        setValue(ColumnField.PRINCIPAL, principal);
    }

    public int getIndexed() {
        return (Integer) getValue(ColumnField.INDEXED);
    }

    public void setIndexed(int indexed) {
        setValue(ColumnField.INDEXED, indexed);
    }

    public int getStd() {
        return (Integer) getValue(ColumnField.STD);
    }

    public void setStd(int std) {
        setValue(ColumnField.STD, std);
    }

    public boolean isChanged(ColumnField key) {
        EditableValue property = properties.get(key);
        if (property != null) {
            return property.isChanged();
        }
        return false;
    }

    public Model getPropertyModel(ColumnField key) {
        return new ColumnPropertyModel(key);
    }
}
